package com.upyuns.platform.rs.website.service;

import com.github.wxiaoqi.security.common.msg.ObjectRestResponse;
import com.upyuns.platform.rs.universal.constant.DictionaryKey;
import com.upyuns.platform.rs.universal.entity.Dictionary;
import com.upyuns.platform.rs.universal.feign.ThirdFeign;
import com.upyuns.platform.rs.website.biz.*;
import com.upyuns.platform.rs.website.dto.ConfirmOrderDto;
import com.upyuns.platform.rs.website.dto.UserAddressDto;
import com.upyuns.platform.rs.website.dto.UserInvoiceDto;
import com.upyuns.platform.rs.website.entity.*;
import com.upyuns.platform.rs.website.type.OrderTypeEnum;
import com.upyuns.platform.rs.website.vo.ConfirmOrderInfoVo;
import lombok.extern.slf4j.Slf4j;
import org.apache.commons.lang3.StringUtils;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;

import java.math.BigDecimal;
import java.util.Arrays;
import java.util.List;
import java.util.Map;
import java.util.Set;
import java.util.stream.Collectors;

/**
 * @ClassName : ConfirmOrderService
 * @Description : 确认订单信息
 * @Author : jiaoruizhen
 * @Date: 2020-12-16 14:50
 */
@Service
@Slf4j
public class ConfirmOrderService {

	@Autowired
	IndustryApplicationInfoBiz industryApplicationInfoBiz;

	@Autowired
	ImageImgStorageBiz imageImgStorageBiz;

	@Autowired
	ShoppingCartInfoBiz shoppingCartInfoBiz;

	@Autowired
	ImageInfoRelationBiz imageInfoRelationBiz;

	@Autowired
	ThirdFeign thirdFeign;

	@Autowired
	UserAddressBiz userAddressBiz;

	@Autowired
	UserInvoiceBiz userInvoiceBiz;

	/**
	 * 确认订单
	 * @param confirmOrderDto
	 * @return
	 */
	public ObjectRestResponse confirmOrderInfo(ConfirmOrderDto confirmOrderDto) {
		if (confirmOrderDto == null || confirmOrderDto.getType() == null || StringUtils.isBlank(confirmOrderDto.getIds())) {
			return ObjectRestResponse.paramIsEmpty();
		}
		String[] idArr = confirmOrderDto.getIds().split(",");
		Set<Integer> idList = Arrays.asList(idArr).stream().map(s -> Integer.parseInt(s.trim())).collect(Collectors.toSet());
		ConfirmOrderInfoVo confirmOrderInfoVo = new ConfirmOrderInfoVo();
		confirmOrderInfoVo.setType(confirmOrderDto.getType());
		confirmOrderInfoVo.setIds(confirmOrderDto.getIds());
		switch (OrderTypeEnum.getByCode(confirmOrderDto.getType())) {
			//标准数据
			case STANDARD_DATA:

				break;
			//行业应用
			case INDUSTRY_INFO:
				List<IndustryApplicationInfo> industryApplicationInfoList = industryApplicationInfoBiz.getAllByIds(idList);
				if (industryApplicationInfoList != null && industryApplicationInfoList.size() > 0) {
					confirmOrderInfoVo.setIndustryApplicationInfoList(industryApplicationInfoList);
					//总数量
					Double amount = industryApplicationInfoList.stream().mapToDouble(e->e.getPrice().doubleValue()).reduce(0, Double::sum);
					confirmOrderInfoVo.setTotalAmount(new BigDecimal(amount));
					confirmOrderInfoVo.setNumber(industryApplicationInfoList.size());
				}
				break;
				//影像图库
			case IMAGE_STORAGE:
				List<ImageInfoRelation> imageInfoRelationList = imageInfoRelationBiz.getAllByIds(confirmOrderDto.getIds());
				if (imageInfoRelationList != null && imageInfoRelationList.size() > 0) {
					Map<Integer, List<ImageInfoRelation>> map = imageInfoRelationList.stream().collect(Collectors.groupingBy(ImageInfoRelation::getRelationId));
					List<ImageImgStorage> imageImgStorageList = imageImgStorageBiz.getAllByIds(imageInfoRelationList.stream().map(ImageInfoRelation::getRelationId).collect(Collectors.toSet()));
					if (imageImgStorageList != null && imageImgStorageList.size() > 0) {
						imageImgStorageList.parallelStream().forEach(imageImgStorage -> {
							if (map != null && map.get(imageImgStorage.getId()) != null) {
								imageImgStorage.setImageInfoRelationList(map.get(imageImgStorage.getId()));
							}
						});
						confirmOrderInfoVo.setImageImgStorageList(imageImgStorageList);
						//总数量
						confirmOrderInfoVo.setNumber(imageInfoRelationList.size());
						Double amount = imageInfoRelationList.stream().mapToDouble(e->e.getPrice().doubleValue()).reduce(0, Double::sum);
						confirmOrderInfoVo.setTotalAmount(new BigDecimal(amount));
					}
				}
				break;
			//购物车
			case SHOPPING_CART_INFO:
				List<ShoppingCartInfo> shoppingCartInfoList = shoppingCartInfoBiz.getAllByIds(idList);
				if (shoppingCartInfoList != null && shoppingCartInfoList.size() > 0) {
					confirmOrderInfoVo.setShoppingCartInfoList(shoppingCartInfoList);
					Double amount = shoppingCartInfoList.stream().mapToDouble(e->e.getTotalAmount().doubleValue()).reduce(0, Double::sum);
					confirmOrderInfoVo.setTotalAmount(new BigDecimal(amount));
				}
				break;
			default:
				break;
		}
		confirmOrderInfoVo.setActAmount(confirmOrderInfoVo.getTotalAmount());

		Dictionary dictionary = thirdFeign.findDictionaryByTypeAndCode(DictionaryKey.APP_ORDER, DictionaryKey.SEND_FEE);
		if (dictionary != null && StringUtils.isBlank(dictionary.getDetail())) {
			confirmOrderInfoVo.setFee(new BigDecimal(dictionary.getDetail()));
		}
		try {
			List<UserAddress> userAddressList = userAddressBiz.getUserAddress(new UserAddressDto(){{setPage(1); setLimit(Integer.MAX_VALUE);}}).getData().getData();
			if (userAddressList != null && userAddressList.size() > 0) {
				confirmOrderInfoVo.setUserAddressList(userAddressList);
			}
		} catch (Exception e) {
			log.error("用户地址信息错误: {}", e);
		}

		try {
			List<UserInvoice> userInvoiceList = userInvoiceBiz.getUserInvoiceInfo(new UserInvoiceDto(){{setPage(1); setLimit(Integer.MAX_VALUE);}}).getData().getData();
			if (userInvoiceList != null && userInvoiceList.size() > 0) {
				confirmOrderInfoVo.setUserInvoiceList(userInvoiceList);
			}
		} catch (Exception e) {
			log.error("用户发票信息错误: {}", e);
		}

		return ObjectRestResponse.succ(confirmOrderInfoVo);
	}

}
